/* ----------------------------------------------------------------------------
  f2cad: Convert Fortran to C++ AD Types: Copyright (C) 2005-10 Bradley M. Bell
  Authors:  Bradley M. Bell    bradbell at washington dot edu
  License:  GNU Public License Version 2 or higher
-----------------------------------------------------------------------------*/
/*
$begin d9knus$$ 
$comment  The omhelp commands are documented in
	http://www.seanet.com/~bradbell/omhelp/
$$
$spell
	cmath
	exp
	std
	atan
	sqrt
	xnu
	bknu
	iswtch
	Bessel
	Abramowitz
	Stegun
	d9knus
	cpp
	hpp
	bool
	doublereal
	adolc
	cppad
	fadbad
$$

$section d9knus$$

$index d9knus$$

$head Prototype$$
$include p.omh/d9knus.p.omh$$

$children%
	f.omh/d9knus.f.omh
%$$
$head Fortran Source$$
$cref d9knus.f$$


$head Notation$$
We use $latex K_{\nu} (x)$$ to denote the modified Bessel function
of the second kind; i.e., it is the solution of the ODE
$latex \[
x^2 y^{(2)} (x) + x y^{(1)} (x) - ( x^2 + \nu^2 ) = 0
\] $$
that goes to zero as $latex x \rightarrow \infty$$.

$head Description$$
The routine $codei d9knus$$ computes the values
$latex \exp(x) K_\nu (x)$$ and $latex \exp(x) K_{\nu+1} (x)$$ 
for $latex x \in [0,1]$$ and $latex \nu \in [0,1]$$.

$head Special Case$$
By Formula 10.2.17 of Abramowitz and Stegun 
$latex \[
	K_{1/2} ( x ) = \sqrt{  \pi / ( 2 x ) } \exp( - x )
\] $$


$codep */

# include <cmath>
# include <f2cad/d9knus.hpp>

test_result d9knus(void)
{	bool ok = true;

	// independent variable
	integer    n = 1;          // number of independent variables
	doublereal x[1];           // vector of independent variables
	x[0]         = .75;        // value of independent variable
	f2cad::Independent(n, x);  // declare x as independent variable vector

	// call the routine
	doublereal xnu = .5;
	doublereal bknu, bknu1;
	integer    iswtch;  
	f2cad::d9knus_(&xnu, x+0, &bknu, &bknu1, &iswtch);

	// dependent variables 
	integer     m = 1;          // number of dependent variables
	doublereal  f[1];           // vector of dependent variables
	f[0] = bknu;                // value f[0] = K_nu (x[0]) * exp( x[0] )
	f2cad::Dependent(m, f);     // declare f as dependent variable vector

	// check function value
	double x0    = f2cad::Value( x[0] );
	double f0    = f2cad::Value( f[0] );
	double pi    = 4. * std::atan(1.);
	double check = 1. / sqrt( 2. * x0 / pi );
	ok          &= f2cad::near_equal(f0, check, 1e-10, 1e-10);

	// Evaluate the derivative of f[0] w.r.t x[0]
	double p = f2cad::Partial<doublereal>(0, 0);
	check    = - check * check * check / pi;
	ok      &= f2cad::near_equal(p, check, 1e-10, 1e-10);

	if( ok ) 
		return test_pass;
	return test_fail;
}
/* $$
$end
*/
