/* ----------------------------------------------------------------------------
  f2cad: Convert Fortran to C++ AD Types: Copyright (C) 2005-10 Bradley M. Bell
  Authors:  Bradley M. Bell    bradbell at washington dot edu
  License:  GNU Public License Version 2 or higher
-----------------------------------------------------------------------------*/
/*
$begin i1mach$$
$spell
	cmlib
	cpp
	nerr
	const
$$

$section Implementation of Cmlib i1mach Routine in Cpp$$

$index i1mach, C++$$
$index C++, i1mach$$

$head Syntax$$ 
$icode%j% = i1mach_(%i%)%$$

$head Purpose$$
This routine uses the C++ standard $code numeric_limits$$ to
implement the specifications required by the cmlib Fortran routine 
$code i1mach$$.

$head i$$
The argument $icode i$$ has prototype
$codei%
	integer* %i%
%$$
It specifies which machine constant to evaluate.

$head d$$
The return value $icode i$$ has prototype
$code%
	integer %d%
%$$
and is the value of the requested machine constant.



$head Mapping$$
The meaning for the machine constants below was
extracted from the comments in the Fortran routine $code i1mach$$.
The constants corresponding to $icode%i% = 1, 2, 3, 4%$$ are not defined.
If any of these constants are requested,
$code i1mach$$ will generate an assertion error.
Also note that, in the $code f2cad$$ library,
single and double precision Fortran values are both represented
using C++ doubles.
Hence, the return values for single and double precision are the same.

$table
$icode i$$ $cnext $code j$$ 
$rnext
1   $cnext standard input unit.
$rnext
2   $cnext standard output unit.
$rnext
3   $cnext standard punch unit.
$rnext
4   $cnext standard error message unit.
$rnext
5   $cnext number of bits per integer storage unit.
$rnext
6   $cnext number of characters per integer storage unit.
$rnext
7   $cnext base used for representation of integers.
$rnext
8   $cnext number of base digits per integer storage unit.
$rnext
9   $cnext largest integer magnitude
$rnext
10  $cnext base for representing floating point numbers.
$rnext
11  $cnext number of base digits in representation of single precision.
$rnext
12  $cnext smallest exponent in representation of single precision. 
$rnext
13  $cnext largest exponent in representation of single precision.
$rnext
14  $cnext number of base digits in representation of double precision.
$rnext
15  $cnext smallest exponent in representation of double precision. 
$rnext
16  $cnext largest exponent in representation of double precision.
$tend

$end
*/

# include <cstdio>
# include <cassert>
# include <cstdlib>

# include <limits>
# include <climits>
# include <cassert>
# include <cmath>

# include <f2cad/f2cad.hpp>


// we should remove the definition of the macros min and max from f2cad.hpp
# undef min
# undef max

namespace f2cad {
	integer i1mach_(integer *i)
	{
		// check some assumptions 
		assert( sizeof(char) == 1 );

		integer return_value;
		switch( *i )
		{
			case 5:
			// i1mach(5) = number of bits per integer storage unit.
			return_value = sizeof(integer) * CHAR_BIT;
			break;
	
			case 6:
			// i1mach(6) = number of characters per integer storage unit.
			return_value = sizeof(integer);
			break;
	
			case 7:
			// i1mach(7) = base used for representation of integers.
			return_value = std::numeric_limits<integer>::radix;
			break;
	
			case 8:
			// i1mach(8) = number of base digits per integer storage unit.
			return_value = std::numeric_limits<integer>::digits;
			break;
	
			case 9:
			// i1mach(9) = largest integer magnitude
			return_value = std::numeric_limits<integer>::max();
			break;
	
			case 10:
			// i1mach(10) = base for representing floating point numbers.
			return_value = std::numeric_limits<double>::radix;
			break;

			case 11:
			case 14:
			// i1mach(14) = number of base digits in double mantissa.
			return_value = std::numeric_limits<double>::digits;
			break;

			case 12:
			case 15:
			// i1mach(15) = smallest exponent in double representation.
			return_value = std::numeric_limits<double>::min_exponent;
			break;

			case 13:
			case 16:
			// i1mach(16) = largest exponent in double representation.
			return_value = std::numeric_limits<double>::max_exponent;
			break;
	
			default:
			assert(false);
			return_value = 0;
		} 	
		return return_value;
	}
}
